@@ -23,8 +23,10 @@
 """Classes for geometry operations"""
 
 import warnings
+from collections import OrderedDict
 
 import numpy as np
+import yaml
 
 from pyresample import _spatial_mp, utils
 
@@ -33,6 +35,10 @@ class DimensionError(Exception):
     pass
 
 
+class IncompatibleAreas(Exception):
+    """Error when the areas to combine are not compatible."""
+
+
 class Boundary(object):
 
     """Container for geometry boundary.
@@ -581,13 +587,36 @@ class AreaDefinition(BaseDefinition):
                     ', '.join(["'%s': '%s'" % (str(k), str(proj_dict[k]))
                                for k in sorted(proj_dict.keys())]) +
                     '}')
-        return ('Area ID: %s\nName: %s\nProjection ID: %s\n'
-                'Projection: %s\nNumber of columns: %s\nNumber of rows: %s\n'
-                'Area extent: %s') % (self.area_id, self.name, self.proj_id,
-                                      proj_str, self.x_size, self.y_size,
-                                      self.area_extent)
+
+        if self.proj_id is None:
+            third_line = ""
+        else:
+            third_line = "Projection ID: {0}\n".format(self.proj_id)
+
+        return ('Area ID: {0}\nDescription: {1}\n{2}'
+                'Projection: {3}\nNumber of columns: {4}\nNumber of rows: {5}\n'
+                'Area extent: {6}').format(self.area_id, self.name, third_line,
+                                           proj_str, self.x_size, self.y_size,
+                                           self.area_extent)
 
     def create_areas_def(self):
+        to_dump = OrderedDict()
+        res = OrderedDict()
+        to_dump[self.area_id] = res
+
+        res['description'] = self.name
+        res['shape'] = OrderedDict([('height', self.y_size),
+                                    ('width', self.x_size)])
+        res['area_extent'] = OrderedDict([('lower_left_xy',
+                                           list(self.area_extent[:2])),
+                                          ('upper_right_xy',
+                                           list(self.area_extent[2:])),
+                                          ('units', 'm')
+                                          ])
+
+        return ordered_dump(to_dump)
+
+    def create_areas_def_legacy(self):
         proj_dict = self.proj_dict
         proj_str = ','.join(["%s=%s" % (str(k), str(proj_dict[k]))
                              for k in sorted(proj_dict.keys())])
@@ -599,8 +628,10 @@ class AreaDefinition(BaseDefinition):
         fmt += "\tXSIZE:\t{x_size}\n"
         fmt += "\tYSIZE:\t{y_size}\n"
         fmt += "\tAREA_EXTENT: {area_extent}\n}};\n"
-        area_def_str = fmt.format(name=self.name, area_id=self.area_id, proj_str=proj_str,
-                                  x_size=self.x_size, y_size=self.y_size, area_extent=self.area_extent)
+        area_def_str = fmt.format(name=self.name, area_id=self.area_id,
+                                  proj_str=proj_str, x_size=self.x_size,
+                                  y_size=self.y_size,
+                                  area_extent=self.area_extent)
         return area_def_str
 
     __repr__ = __str__
@@ -674,29 +705,9 @@ class AreaDefinition(BaseDefinition):
                 raise ValueError("lon and lat is not of the same shape!")
 
         pobj = _spatial_mp.Proj(self.proj4_string)
-        upl_x = self.area_extent[0]
-        upl_y = self.area_extent[3]
-        xscale = abs(self.area_extent[2] -
-                     self.area_extent[0]) / float(self.x_size)
-        yscale = abs(self.area_extent[1] -
-                     self.area_extent[3]) / float(self.y_size)
-
         xm_, ym_ = pobj(lon, lat)
-        x__ = (xm_ - upl_x) / xscale
-        y__ = (upl_y - ym_) / yscale
 
-        if isinstance(x__, np.ndarray) and isinstance(y__, np.ndarray):
-            mask = (((x__ < 0) | (x__ > self.x_size)) |
-                    ((y__ < 0) | (y__ > self.y_size)))
-            return (np.ma.masked_array(x__.astype('int'), mask=mask,
-                                       fill_value=-1),
-                    np.ma.masked_array(y__.astype('int'), mask=mask,
-                                       fill_value=-1))
-        else:
-            if ((x__ < 0 or x__ > self.x_size) or
-                    (y__ < 0 or y__ > self.y_size)):
-                raise ValueError('Point outside area:( %f %f)' % (x__, y__))
-            return int(x__), int(y__)
+        return self.get_xy_from_proj_coords(xm_, ym_)
 
     def get_xy_from_proj_coords(self, xm_, ym_):
         """Retrieve closest x and y coordinates (column, row indices) for a
@@ -732,21 +743,22 @@ class AreaDefinition(BaseDefinition):
 
         upl_x = self.area_extent[0]
         upl_y = self.area_extent[3]
-        xscale = abs(self.area_extent[2] -
-                     self.area_extent[0]) / float(self.x_size)
-        yscale = abs(self.area_extent[1] -
-                     self.area_extent[3]) / float(self.y_size)
+        xscale = (self.area_extent[2] -
+                  self.area_extent[0]) / float(self.x_size)
+        # because rows direction is the opposite of y's
+        yscale = (self.area_extent[1] -
+                  self.area_extent[3]) / float(self.y_size)
 
         x__ = (xm_ - upl_x) / xscale
-        y__ = (upl_y - ym_) / yscale
+        y__ = (ym_ - upl_y) / yscale
 
         if isinstance(x__, np.ndarray) and isinstance(y__, np.ndarray):
             mask = (((x__ < 0) | (x__ > self.x_size)) |
                     ((y__ < 0) | (y__ > self.y_size)))
             return (np.ma.masked_array(x__.astype('int'), mask=mask,
-                                       fill_value=-1),
+                                       fill_value=-1, copy=False),
                     np.ma.masked_array(y__.astype('int'), mask=mask,
-                                       fill_value=-1))
+                                       fill_value=-1, copy=False))
         else:
             if ((x__ < 0 or x__ > self.x_size) or
                     (y__ < 0 or y__ > self.y_size)):
@@ -934,7 +946,7 @@ class AreaDefinition(BaseDefinition):
             dtype = self.dtype
 
         if self.lons is None or self.lats is None:
-            #Data is not cached
+            # Data is not cached
             if nprocs is None:
                 nprocs = self.nprocs
 
@@ -963,7 +975,7 @@ class AreaDefinition(BaseDefinition):
             del(target_x)
             del(target_y)
         else:
-            #Data is cached
+            # Data is cached
             if data_slice is None:
                 # Full slice
                 lons = self.lons
@@ -982,6 +994,137 @@ class AreaDefinition(BaseDefinition):
         return '+' + ' +'.join([t[0] + '=' + str(t[1]) for t in items])
 
 
+def combine_area_extents_vertical(area1, area2):
+    """Combine the area extents of areas 1 and 2."""
+    if (area1.area_extent[0] == area2.area_extent[0] and
+            area1.area_extent[2] == area2.area_extent[2]):
+        current_extent = list(area1.area_extent)
+        if np.isclose(area1.area_extent[1], area2.area_extent[3]):
+            current_extent[1] = area2.area_extent[1]
+        elif np.isclose(area1.area_extent[3], area2.area_extent[1]):
+            current_extent[3] = area2.area_extent[3]
+    else:
+        raise IncompatibleAreas(
+            "Can't concatenate area definitions with "
+            "incompatible area extents: "
+            "{0} and {1}".format(area1, area2))
+    return current_extent
+
+
+def concatenate_area_defs(area1, area2, axis=0):
+    """Append *area2* to *area1* and return the results"""
+    different_items = (set(area1.proj_dict.items()) ^
+                       set(area2.proj_dict.items()))
+    if axis == 0:
+        same_size = area1.x_size == area2.x_size
+    else:
+        raise NotImplementedError('Only vertical contatenation is supported.')
+    if different_items or not same_size:
+        raise IncompatibleAreas("Can't concatenate area definitions with "
+                                "different projections: "
+                                "{0} and {1}".format(area1, area2))
+
+    if axis == 0:
+        area_extent = combine_area_extents_vertical(area1, area2)
+        x_size = area1.x_size
+        y_size = area1.y_size + area2.y_size
+    else:
+        raise NotImplementedError('Only vertical contatenation is supported.')
+    return AreaDefinition(area1.area_id, area1.name, area1.proj_id,
+                          area1.proj_dict, x_size, y_size,
+                          area_extent)
+
+
+class StackedAreaDefinition(BaseDefinition):
+    """Definition based on muliple vertically stacked AreaDefinitions."""
+
+    def __init__(self, *definitions, **kwargs):
+        """Base this instance on *definitions*.
+
+        *kwargs* used here are `nprocs` and `dtype` (see AreaDefinition).
+        """
+        nprocs = kwargs.get('nprocs', 1)
+        super(StackedAreaDefinition, self).__init__(nprocs=nprocs)
+        self.dtype = kwargs.get('dtype', np.float64)
+        self.defs = []
+        self.proj_dict = {}
+        for definition in definitions:
+            self.append(definition)
+
+    @property
+    def x_size(self):
+        return self.defs[0].x_size
+
+    @property
+    def y_size(self):
+        return sum(definition.y_size for definition in self.defs)
+
+    @property
+    def size(self):
+        return self.y_size * self.x_size
+
+    def append(self, definition):
+        """Append another definition to the area."""
+        if isinstance(definition, StackedAreaDefinition):
+            for area in definition.defs:
+                self.append(area)
+            return
+        if definition.y_size == 0:
+            return
+        if not self.defs:
+            self.proj_dict = definition.proj_dict
+        elif self.proj_dict != definition.proj_dict:
+            raise NotImplementedError('Cannot append areas:'
+                                      ' Proj.4 dict mismatch')
+        try:
+            self.defs[-1] = concatenate_area_defs(self.defs[-1], definition)
+        except (IncompatibleAreas, IndexError):
+            self.defs.append(definition)
+
+    def get_lonlats(self, nprocs=None, data_slice=None, cache=False, dtype=None):
+        """Return lon and lat arrays of the area."""
+
+        llons = []
+        llats = []
+        try:
+            row_slice, col_slice = data_slice
+        except TypeError:
+            row_slice = slice(0, self.y_size)
+            col_slice = slice(0, self.x_size)
+        offset = 0
+        for definition in self.defs:
+            local_row_slice = slice(max(row_slice.start - offset, 0),
+                                    min(max(row_slice.stop - offset, 0),
+                                        definition.y_size),
+                                    row_slice.step)
+            lons, lats = definition.get_lonlats(nprocs=nprocs,
+                                                data_slice=(local_row_slice,
+                                                            col_slice),
+                                                cache=cache,
+                                                dtype=dtype)
+
+            llons.append(lons)
+            llats.append(lats)
+            offset += lons.shape[0]
+
+        self.lons = np.vstack(llons)
+        self.lats = np.vstack(llats)
+
+        return self.lons, self.lats
+
+    def squeeze(self):
+        """Generate a single AreaDefinition if possible."""
+        if len(self.defs) == 1:
+            return self.defs[0]
+        else:
+            return self
+
+    @property
+    def proj4_string(self):
+        """Returns projection definition as Proj.4 string"""
+        return self.defs[0].proj4_string
+
+
 def _get_slice(segments, shape):
     """Generator for segmenting a 1D or 2D array"""
 
@@ -1024,3 +1167,16 @@ def _get_highest_level_class(obj1, obj2):
     else:
         klass = obj1.__class__
     return klass
+
+
+def ordered_dump(data, stream=None, Dumper=yaml.Dumper, **kwds):
+    class OrderedDumper(Dumper):
+        pass
+
+    def _dict_representer(dumper, data):
+        return dumper.represent_mapping(
+            yaml.resolver.BaseResolver.DEFAULT_MAPPING_TAG,
+            data.items(), flow_style=False)
+
+    OrderedDumper.add_representer(OrderedDict, _dict_representer)
+    return yaml.dump(data, stream, OrderedDumper, **kwds)
