@@ -24,7 +24,19 @@
 """
 
 
-from pyresample.test import test_geometry, test_grid, test_image, test_kd_tree, test_plot, test_spherical_geometry, test_swath, test_utils
+from pyresample.test import (
+    test_geometry,
+    test_grid,
+    test_image,
+    test_kd_tree,
+    test_plot,
+    test_spherical_geometry,
+    test_swath,
+    test_utils,
+    test_ewa_ll2cr,
+    test_ewa_fornav,
+    test_bilinear,
+)
 
 import unittest
 
@@ -42,5 +54,12 @@ def suite():
     mysuite.addTests(test_spherical_geometry.suite())
     mysuite.addTests(test_swath.suite())
     mysuite.addTests(test_utils.suite())
+    mysuite.addTests(test_ewa_ll2cr.suite())
+    mysuite.addTests(test_ewa_fornav.suite())
+    mysuite.addTests(test_bilinear.suite())
 
     return mysuite
+
+
+def load_tests(loader, tests, pattern):
+    return suite()
